Package: credentials
Type: Package
Title: Tools for Managing SSH and Git Credentials
Version: 2.0.2
Authors@R: person("Jeroen", "Ooms", role = c("aut", "cre"), 
    email = "jeroenooms@gmail.com", comment = c(ORCID = "0000-0002-4035-0289"))
Description: Setup and retrieve HTTPS and SSH credentials for use with 'git' and 
    other services. For HTTPS remotes the package interfaces the 'git-credential' 
    utility which 'git' uses to store HTTP usernames and passwords. For SSH 
    remotes we provide convenient functions to find or generate appropriate SSH 
    keys. The package both helps the user to setup a local git installation, and
    also provides a back-end for git/ssh client libraries to authenticate with 
    existing user credentials.
License: MIT + file LICENSE
SystemRequirements: git (optional)
Encoding: UTF-8
Imports: openssl (>= 1.3), sys (>= 2.1), curl, jsonlite, askpass
Suggests: testthat, knitr, rmarkdown
RoxygenNote: 7.2.1
VignetteBuilder: knitr
Language: en-US
URL: https://docs.ropensci.org/credentials/
        https://r-lib.r-universe.dev/credentials
BugReports: https://github.com/r-lib/credentials/issues
NeedsCompilation: no
Packaged: 2024-10-03 14:12:32 UTC; jeroen
Author: Jeroen Ooms [aut, cre] (<https://orcid.org/0000-0002-4035-0289>)
Maintainer: Jeroen Ooms <jeroenooms@gmail.com>
Repository: CRAN
Date/Publication: 2024-10-04 12:40:03 UTC
Built: R 4.3.3; ; 2025-01-24 17:59:13 UTC; unix
