# Generated by roxygen2: do not edit by hand

export(credential_approve)
export(credential_fill)
export(credential_helper_get)
export(credential_helper_list)
export(credential_helper_set)
export(credential_reject)
export(git_credential_ask)
export(git_credential_forget)
export(git_credential_update)
export(set_github_pat)
export(ssh_agent_add)
export(ssh_home)
export(ssh_key_info)
export(ssh_keygen)
export(ssh_read_key)
export(ssh_setup_github)
export(ssh_update_passphrase)
importFrom(askpass,askpass)
importFrom(askpass,ssh_askpass)
importFrom(openssl,read_key)
importFrom(openssl,read_pubkey)
importFrom(openssl,write_pem)
importFrom(openssl,write_pkcs1)
importFrom(openssl,write_ssh)
